
package scala.tools.nsc
package backend
package icode

import scala.tools.nsc.{plugins, Global, Phase}
import plugins.PluginComponent
import java.io.{PrintWriter, FileOutputStream, File}

object GenCSharpPlugin
{
	val PluginName = "gencsharp"
	/** This is the name of the option that specifies the base directory against which sources
   * should be relativized.*/
	val BaseDirectoryOptionName = "base-directory:"

}

class GenCSharpPlugin(val global: Global) extends GenCSharp
{
	import global._
	import GenCSharpPlugin._

	val name = PluginName
	val description = "A plugin to generate a tree of C# source files."
	lazy val components = List[PluginComponent](Component)

	/** The directory to which the annotated sources will be written. */
	val outputDirectory = {
	  val f = new File(settings.outdir.value)
	  new File(f.getParent, f.getName).getAbsoluteFile
  }
	outputDirectory.mkdirs()

	/** The directory against which the input source paths will be relativized.*/
	var baseDirectory: Option[File] = None

  var afterWhichPhase = "icode"

	override def processOptions(options: List[String], error: String => Unit)
	{
		for(option <- options)
		{
			if(option.startsWith(BaseDirectoryOptionName))
				baseDirectory = Some(new File(option.substring(BaseDirectoryOptionName.length)))

		}
    if(baseDirectory == None)
      error("Base directory option for stst plugin not understood.")
	}
	override val optionsHelp: Option[String] =
	{
		val prefix = "  -P:" + name + ":"
    val msg1 = prefix + BaseDirectoryOptionName + "<name>            Set the base source directory .\n"
		Some(msg1)
	}

	/* For source compatibility between 2.7.x and 2.8.x */
	private object runsBefore { def :: (s: String) = s }
	private abstract class CompatiblePluginComponent(afterPhase: String) extends PluginComponent
	{
		val runsAfter = afterPhase :: runsBefore
	}
	private object Component extends CompatiblePluginComponent(afterWhichPhase)
	{
		val global = GenCSharpPlugin.this.global
		val phaseName = GenCSharpPlugin.this.name
		def newPhase(prev: Phase) = new GenCSharpPhase(prev)
	}

  /** Relativizes the path to the given Scala source file to the base directory. */
  private def getRelativeSourcePath(source: File): String =
  {
    baseDirectory match
    {
      case Some(base) =>
        FileUtil.relativize(base, source) match
        {
          case Some(relative) => relative
          case None => error("Source " + source + " not in base directory " + base); ""
        }
      case None => source.getName
    }
  }

  /** The entry method for producing a tree of C# files */
  def generateOutput()
  {
    var outputFiles = List[File]()
    val msilLinearizer = new icodes.MSILLinearizer()
    for(unit <- currentRun.units)
    {
      val sourceFile = unit.source.file.file
      var relativeSourcePath = getRelativeSourcePath(sourceFile)
      if(relativeSourcePath.endsWith(".scala")) {
        val upTo = relativeSourcePath.length - (".scala".length)
        relativeSourcePath = relativeSourcePath.substring(0, upTo) + ".cs"
      } else {
        relativeSourcePath += ".cs"
      }
      val outputFile = new File(outputDirectory, relativeSourcePath)
      outputFile.getParentFile.mkdirs()
      val ofs = new FileOutputStream(outputFile)
      outputFiles ::= outputFile

      val pw = new PrintWriter(ofs, true)
      val printer = new CSharpTextPrinter(pw, msilLinearizer)

      for (cls <- icodes.classes.values if cls.cunit == unit) {
        pw.println("/*#########################################################*/")
        printer.printClass(cls) // TODO print here the 3-address form instead
        pw.println
      }

    }
  }


}